(in-package :http)

(add-lispweb-service :javascript
		     :description "Library of JavaScript(tm) functions.")

(defconstant *LispWeb-JS-library*
    '(*js-is-empty*
      *js-is-date*
      *js-is-number*
      *js-in-bounds*
      *js-radio-checked*
      *js-current-time*
      *js-goto-url*
      *js-new-window*
      *js-set-status*
      js-window-options
      js-update-form
      *js-multijump*))

(eval-when (load eval)
  (export *LispWeb-JS-library* :http)
  (export *LispWeb-JS-library* :lispweb-user))

(defconstant *js-is-empty*
    "function is_empty(value)
{
  return ((value == null) || (value == \"\"));
}

" "Return true if value is not empty")
  
(defconstant *js-is-date*
    "function is_date(value)
{
  return (Date.parse(value)!=0);
}

")

(defconstant *js-is-number*
    "function is_number(value)
{
  digits = \"0123456789+-.\"
    
    if (!is_empty(value)) {
      for (var i=0; i < value.length; i++)
	if (digits.indexOf(value.charAt(i), 0) == -1)
	  return false;
      return true}     
    else
      return false;
}

")

(defconstant *js-in-bounds*
    "function in_bounds(value, min, max) {
  var i = 0;

  if (is_number(value)) {
    i=parseInt(value, 10);
    return ((i >= min) && (i <= max)) }
  else
    return false;
}

")

(defconstant *js-radio-checked*
    "function radio_checked(radio,message){
  var flag = false;
  
  for (var i=0; i < radio.length; i++)
    flag=(flag || radio[i].checked);
  return flag;
}

")

(defconstant *js-current-time*
"function showtime() {
  var now = new Date()
  var hours = now.getHours()
  var minutes = now.getMinutes()
  var seconds = now.getSeconds()
  var timeValue = \"\" + ((hours > 12) ? hours - 12 : hours)
  timeValue  += ((minutes < 10) ? \":0\" : \":\") + minutes
  timeValue  += ((seconds < 10) ? \":0\" : \":\") + seconds
  timeValue  += (hours >= 12) ? \" P.M.\" : \" A.M.\"
  return timeValue;
}

")

(defconstant *js-goto-url*
    "function goto_url(url) {
  parent.window.location = url;
}
")

(defconstant *js-new-window*
"function new_window(page, close, title, options) {
  var win = open(page, title, options);
  if (close) {
    self.close(); 
  return win}
}
")

(defconstant *js-set-status*
"function set_status(message) {
  self.status = message;
}")

(defun js-window-options (&key (toolbar t)
			       (location t)
			       (directories nil)
			       (status t)
			       (menubar t)
			       (scrollbars t)
			       (resizable t)
			       (copyhistory t)
			       (width 1)
			       (height 1))
  (format nil "toolbar=~:[0~;1~],location=~:[0~;1~],directories=~:[0~;1~],status=~:[0~;1~],menubar=~:[0~;1~],scrollbars=~:[0~;1~],resizable=~:[0~;1~],copyhistory=~:[0~;1~],width=~d,height=~d"
	  toolbar
	  location
	  directories
	  status
	  menubar
	  scrollbars
	  resizable
	  copyhistory
	  width
	  height))

(defun js-update-form (function-name select-name fields matrix)
  (with-output-to-string (s)
    (flet ((initialize-arrays ()
	     (let ((arrays (mapcar #'(lambda (x)
				       (declare (ignore x))
				       (gentemp select-name))
				   fields))
		   (cnt -1))
	       (mapcar #'(lambda (arr)
			   (format s "~a = new Array()~%" arr))
		       arrays)
	       (mapcar #'(lambda (field)
			   (incf cnt)
			   (mapcar #'(lambda (array value)
				       (format s "~a[~d] = ~s~%" array cnt value))
				   arrays field))
		       matrix)
	       arrays)))
      (let ((names (initialize-arrays)))
	(format s "~%function ~a(sel) {~%" function-name)
	;(format s "  formname=this.form~%")
	(format s "  selectInteger=sel.selectedIndex~%")
	(mapc #'(lambda (field name)
		  (format s "  sel[\"form\"].~a.value = ~a[selectInteger]~%"
			  field
			  name))
	      fields names)
	(format s "}~%")))))

(defconstant *js-multijump* 
  "function multijump() {
  for (i=0; i < multijump.arguments.length; i++) {
    if (multijump.arguments[i] != '')
      parent.window.frames[i].location = multijump.arguments[i]
  }
}~%")

