(in-package :http)

;;; The argument list has the following format:
;;; (option-name controlled-variable &optional numberp documentation)
;;; The documentation is needed only for the last variant of an option.

(defvar *arguments-table*
  '(("-hp" *http-port* t)
    ("-http-port" *http-port* t "TCP port number for incoming HTTP connections.")
    ("-ep" *eval-port* t)
    ("-eval-port" *eval-port* t "TCP port number for incoming connections to the Listener.")
    ("-rdir" *LispWeb-root-dir* nil "Root dir of the server file-system.")
    ("-cdir" *LispWeb-conf-dir* nil "Root dir of the server file-system.")
    ("-i" *LispWeb-init-file*)
    ("-init" *LispWeb-init-file* nil "File to load at server startup.")
    ("-lf" *LispWeb-log-file*)
    ("-log-file" *LispWeb-log-file* nil "Server log filename.")
    ("-df" *LispWebdebug-file*)
    ("-debug-file" *LispWeb-debug-file* nil "Server debug filename.")
    ("-tf" *LispWeb-timing-file*)
    ("-timing-file" *LispWeb-timing-file* nil "Server timing info filename.")
    ("-hd" *http-delay*)
    ("-http-delay" *http-delay* t "Number of seconds to sleep between polls.")
    ("-m" *http-multithreaded-p*)
    ("-multi" *http-multithreaded-p* nil "Handle multiple requests in parallel?")
    ("-oa" *operator-address*)
    ("-op-add" *operator-address* nil "Email address of the server operator.")
    ("-op" *operator-password*)
    ("-op-pass" *operator-password* nil "Password to use the control page and the Network Listener.")))

(defun parse-arguments ()
  "Interpret command-line arguments and return a list of supplied options."
  (let ((index 0)			;skip image name
	(argument nil)
	(option nil)
	(value nil)
	(option-list nil))
  (loop 
    (incf index)
    (if (>= index (system:command-line-argument-count)) ;no more arguments?
	(return option-list)		;return options
      (setq argument (system:command-line-argument index)))
    ;(format t "Index = ~d, argument = ~s~%" index argument)
    (when (or (string= argument "-h")	;help?
	      (string= argument "-help"))
      (print-help)
      (return :quit))
    (setq option (assoc argument *arguments-table* :test #'string=)) ;look for an option
    (unless option			;undefined?
      (print-help)
      (return :quit))
    ;(format t "Index = ~d, option = ~s~%" index option)
    (incf index)
    (unless (setq value (system:command-line-argument index)) ;look for option value
      (print-help)
      (return :quit))
    ;(format t "Index = ~d, value = ~s~%" index value)
    (if (third option) 
	(push (read-from-string value) option-list) ;push a numeric value...
      (push value option-list))		;... or a string value...
    (push (second option) option-list)))) ;... and the controlled variable.
   
(defun print-help ()
  (format t "~%~%Lisp-http 1.0~%Valid command-line options are:~%~%")
  (mapc #'(lambda (option)
	    (format t "~a~:[, ~;~20T~:*~a~%~]" (car option) (fourth option)))
	*arguments-table*)
  (values))

(defun make-executable (&optional (name "LispWeb"))
  (setq http::*LispWeb-build-date* (http::http-time))

  
  (setq excl::*restart-app-function*	;excl::*restart-function* in beta
    #'(lambda ()
	(let ((options (parse-arguments)))
	  (when (listp options)
	    (start-tcp-services options)
	    (mp:process-wait "Waiting for server shutdown" 
			     #'(lambda ()
				 *closing-server*))))
	(exit))
    excl::*read-init-files* nil)
  (gc)
  (excl:dumplisp :name name))

;;; Alias to start tcp services

(tpl:alias ("tcp" 2) (&optional conf-dir)
  (if conf-dir
      (start-tcp-services (list '*LispWeb-conf-dir* conf-dir))
    (start-tcp-services)))
		       
