(in-package :http)

;;; Generalized log facility

(defvar *LispWeb-log-streams* nil)

(defun lw-set-log (what where)
  "Direct logging of the WHAT subsystem to WHERE, that can be:
T, NIL, a stream, a string or the keyword :here, meaning the
current value of *terminal-io*."
  (let ((old-where (getf *LispWeb-log-streams* what)))
    (maybe-close-log-stream old-where)
    (cond ((stringp where)
	   (setq where (open (merge-pathnames where *LispWeb-conf-dir*)
			     :direction :output
			     :if-does-not-exist :create
			     :if-exists :append)))
	  ((pathnamep where)
	   (setq where (open where
			     :direction :output
			     :if-does-not-exist :create
			     :if-exists :append)))
	  ((eq where :here)
	   (setq where *terminal-io*)))
    (setf (getf *LispWeb-log-streams* what) where)))

(defmethod maybe-close-log-stream (stream)
  (declare (ignore stream))
  nil)

(defmethod maybe-close-log-stream ((stream file-stream))
  (close stream)
  t)

;;;comment out this function if you want to speed things up
(defun lw-log (what message &rest arguments)
  "Format MESSAGE and ARGUMENTS, and send the resulting string to the
log stream for WHAT"
  (let ((stream (getf *LispWeb-log-streams* what)))
    (when (or (eq stream t)
	      (null stream)
	      (streamp stream))
      (handler-case 
	  (progn 
	    (apply #'format stream message arguments)
	    (when (streamp stream)
	      (force-output stream)))
	(stream-error (c)
	              (declare (ignore c))
	              (lw-set-log what nil)))))
  )

(defun log-summary (logfile &key (path "") domain)
  (let ((line nil)
	(machine nil)
	(count 0)
	(machines nil)
	(start-date nil)
	(end-date nil))
    (with-open-file (lf logfile)
      (loop
	(unless (setq line (read-line lf nil nil))
	  (return))
	(incf count)
	(when (search path line :test #'equalp)
	  (setq machine (read-from-string line))
	  (when domain
	    (setq machine (read-from-string (address-domain (string machine)))))
	  (let ((machine-count (assoc machine machines))
		(date (subseq line 
			      (1+ (position #\[ line))
			      (position #\] line))))
	    (if machine-count
		(incf (cdr machine-count))
	      (push (cons machine 1) machines))
	    (unless start-date
	      (setq start-date date))
	    (setq end-date date)))))
    (format t "~d requests, from ~a to ~a.~%" count start-date end-date)
    (mapc #'(lambda (mc)
	      (format t "~6d  ~(~a~)~%" (cdr mc) (car mc)))
	  (sort machines #'> :key #'cdr))
    (values)))

(defun address-domain (address)
  (if (every #'(lambda (c)
		 (member c '(#\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9 #\.)))
	     address)
      address
    (let ((last-dot (position #\. address :from-end t)))
      (if last-dot
	  (let ((second-dot (position #\. address :from-end t :end last-dot)))
	    (if second-dot
		(subseq address (1+ second-dot)) ;two or more dots
	      address))			;one dot
	address))))			;two dots