(in-package :http)


;;;i'll redefine this 
#|(defmacro html-out (format-string &rest arguments)
  (unless (stringp format-string)
    (push format-string arguments)
    (setq format-string "~a"))
  (let ((format-fun (macroexpand (list 'formatter format-string)))
        (format-args (mapcar #'simplify-arg arguments)))
    `(funcall ,format-fun html-stream ,@format-args)))|# 

(defmacro html-out (format-string &rest arguments)
   (unless (stringp format-string)
    (push format-string arguments)
    (setq format-string "~a"))
  `(format html-stream ,format-string ,@arguments))


(defun simplify-arg (argument)
  (if (and (listp argument)
           (symbolp (car argument))
           (getf (symbol-plist (car argument)) :lispweb-arglist))
      (let ((new-args (mapcar #'simplify-arg (cdr argument))))
	(if (every #'constant-string-p new-args)
	    (apply (car argument) new-args)
	  (cons (car argument) new-args)))
    argument))

(defun constant-string-p (x)
  (when (listp x)
    (setq x (simplify-arg x)))
  (or (keywordp x)
      (and (not (symbolp x))
	   (not (listp x)))))

;;;like define-page but with no header stuff


;;vlad: adds a xml header to define-page2
(defmacro define-page3 (&whole definition
			      (title &key (func-name (gentemp "PAGE-"))
					  (url (format nil "/~(~a~)" func-name))
					  code
					  cookie
					  new-key
					  javascript
					  auth
					  (class :user)
					  style
					  (visible *LispWeb-visible-pages*)
                                          (header-p nil))
			      lambda-list &body forms)
  "This macro defines a function to build a WWW page and registers it.
title       - a string that names the page
func-name   - the name of the defined function
url         - the url of the function 
code        - the http response code associated with the page
meta        - list of meta fields in header
frames      - a frameset specifier
javascript  - JavaScript code to be included in the document's head
package     - the package in which the function should run
background  - background image
base        - base href
bgcolor,
text,
link,
vlink,
alink       - colors
auth        - authorization list for the page
class       - class of the page, :user, :system, or a class name
style       - a page-style structure
visible     - flag for source recording
lambda-list - parameters of the defined function
header-p    - if true the header is printed
forms       - body of the function"
  title
  (let ((decl nil))
    (if (eq (caar forms) 'declare)	;prepare declarations
	(setq decl (append (car forms) (list *lispweb-declarations*))
	      forms (cdr forms))
      (setq decl (list 'declare *lispweb-declarations*)))
    (when visible
      (setq forms (append forms (list `(html-out (source-page-button ',func-name))))))
    (when (keyed-class-p class)
      (if new-key
	  (setq lambda-list
	    (cons '&optional
		  (cons 'http::lispweb-page-key lambda-list)))
	(push 'http::lispweb-page-key lambda-list)))
    `(progn
       (set-symbol-info ',func-name :page ,url ',class)
       ,(when visible
	  `(setf (get ',func-name :hall-source) ',definition))
       (defun ,func-name ,lambda-list
	 ,decl
	 ,(when (keyed-class-p class)	
	    (if new-key
		`(setq http::lispweb-page-key (http::access-key-string (http::new-key ,@new-key)))
	      `(setq http::lispweb-page-key 
		 (test-access-key http::lispweb-page-key)))) ;this does not return if key check fails
	 (let ((code ,(or code
			  (and style `(or (page-style-code ,style) :ok))
			  :ok))
	       (cookie ,(or cookie
			    (and style `(page-style-cookie ,style))))
	       (auth ,(or auth
			  (and style `(page-style-auth ,style))))
	       (*LispWeb-html-indent* 0))
	   (when auth
	     (check-auth auth))		;this does not return if auth check fails
	   (when (page-traced-p ',func-name)
	     (print-page-trace ',func-name))
	   (let ((the-page 
		  (if (service-loaded-p :hall)
		      (with-output-to-string (html-stream)
			(declare (special html-stream))
                        (html-out "<?xml version=\"1.0\"?>~%")
			,(when javascript 
			   `(let ((jscript ,javascript))
			      (when (listp jscript)
				(setq jscript (apply #'concatenate 'string jscript)))
			      (html-out "~a~%" (javascript jscript))))
			,@forms)
		    "")))
             (when ,header-p
	       (http-header *http-xml-header* :code-name code :length (length the-page)
			    :other-headers (when cookie (cookie-field cookie))))
	     (princ the-page *http-stream*)
	     t)))			;end of defun
	 
       ,(when url
	  `(register-page ,url ',func-name ',class))
       ',func-name)))


(defmacro define-page2 (&whole definition
			      (title &key (func-name (gentemp "PAGE-"))
					  (url (format nil "/~(~a~)" func-name))
					  code
					  cookie
					  new-key
					  javascript
					  auth
					  (class :user)
					  style
					  (visible *LispWeb-visible-pages*)
                                          (header-p nil))
			      lambda-list &body forms)
  "This macro defines a function to build a WWW page and registers it.
title       - a string that names the page
func-name   - the name of the defined function
url         - the url of the function 
code        - the http response code associated with the page
meta        - list of meta fields in header
frames      - a frameset specifier
javascript  - JavaScript code to be included in the document's head
package     - the package in which the function should run
background  - background image
base        - base href
bgcolor,
text,
link,
vlink,
alink       - colors
auth        - authorization list for the page
class       - class of the page, :user, :system, or a class name
style       - a page-style structure
visible     - flag for source recording
lambda-list - parameters of the defined function
header-p    - if true the header is printed
forms       - body of the function"
  title
  (let ((decl nil))
    (if (eq (caar forms) 'declare)	;prepare declarations
	(setq decl (append (car forms) (list *lispweb-declarations*))
	      forms (cdr forms))
      (setq decl (list 'declare *lispweb-declarations*)))
    (when visible
      (setq forms (append forms (list `(html-out (source-page-button ',func-name))))))
    (when (keyed-class-p class)
      (if new-key
	  (setq lambda-list
	    (cons '&optional
		  (cons 'http::lispweb-page-key lambda-list)))
	(push 'http::lispweb-page-key lambda-list)))
    `(progn
       (set-symbol-info ',func-name :page ,url ',class)
       ,(when visible
	  `(setf (get ',func-name :hall-source) ',definition))
       (defun ,func-name ,lambda-list
	 ,decl
	 ,(when (keyed-class-p class)	
	    (if new-key
		`(setq http::lispweb-page-key (http::access-key-string (http::new-key ,@new-key)))
	      `(setq http::lispweb-page-key 
		 (test-access-key http::lispweb-page-key)))) ;this does not return if key check fails
	 (let ((code ,(or code
			  (and style `(or (page-style-code ,style) :ok))
			  :ok))
	       (cookie ,(or cookie
			    (and style `(page-style-cookie ,style))))
	       (auth ,(or auth
			  (and style `(page-style-auth ,style))))
	       (*LispWeb-html-indent* 0))
	   (when auth
	     (check-auth auth))		;this does not return if auth check fails
	   (when (page-traced-p ',func-name)
	     (print-page-trace ',func-name))
	   (let ((the-page 
		  (if (service-loaded-p :hall)
		      (with-output-to-string (html-stream)
			(declare (special html-stream))
			,(when javascript 
			   `(let ((jscript ,javascript))
			      (when (listp jscript)
				(setq jscript (apply #'concatenate 'string jscript)))
			      (html-out "~a~%" (javascript jscript))))
			,@forms)
		    "")))
             (when ,header-p
	       (http-header *http-text-header* :code-name code :length (length the-page)
			    :other-headers (when cookie (cookie-field cookie))))
	     (princ the-page *http-stream*)
	     t)))			;end of defun
	 
       ,(when url
	  `(register-page ,url ',func-name ',class))
       ',func-name)))

(defmacro define-page (&whole definition
			      (title &key (func-name (gentemp "PAGE-"))
					  (url (format nil "/~(~a~)" func-name))
					  code
					  meta
					  cookie
					  new-key
					  frames
					  javascript
                                          javascript-source
					  background
					  base
                                          external-base
					  bgcolor
					  text
					  link
					  vlink
					  alink
					  onload
					  onunload
					  auth
                                          left-margin
					  (class :user)
					  style
					  (visible *LispWeb-visible-pages*)
                                          (header-p t))
			      lambda-list &body forms)
  "This macro defines a function to build a WWW page and registers it.
title       - a string that names the page
func-name   - the name of the defined function
url         - the url of the function 
code        - the http response code associated with the page
meta        - list of meta fields in header
frames      - a frameset specifier
javascript  - JavaScript code to be included in the document's head
package     - the package in which the function should run
background  - background image
base        - base href
bgcolor,
text,
link,
vlink,
alink       - colors
auth        - authorization list for the page
class       - class of the page, :user, :system, or a class name
style       - a page-style structure
visible     - flag for source recording
lambda-list - parameters of the defined function
header-p    - if true the header is printed
forms       - body of the function"
  
  (let ((decl nil))
    (if (eq (caar forms) 'declare)	;prepare declarations
	(setq decl (append (car forms) (list *lispweb-declarations*))
	      forms (cdr forms))
      (setq decl (list 'declare *lispweb-declarations*)))
    (when visible
      (setq forms (append forms (list `(html-out (source-page-button ',func-name))))))
    (when (keyed-class-p class)
      (if new-key
	  (setq lambda-list
	    (cons '&optional
		  (cons 'http::lispweb-page-key lambda-list)))
	(push 'http::lispweb-page-key lambda-list)))
    `(progn
       (set-symbol-info ',func-name :page ,url ',class)
       ,(when visible
	  `(setf (get ',func-name :hall-source) ',definition))
       (defun ,func-name ,lambda-list
	 ,decl
	 ,(when (keyed-class-p class)	
	    (if new-key
		`(setq http::lispweb-page-key (http::access-key-string (http::new-key ,@new-key)))
	      `(setq http::lispweb-page-key 
		 (test-access-key http::lispweb-page-key)))) ;this does not return if key check fails
	 (let ((true-base ,(or base
			       (when url `(url ',func-name))))
               (external-base ,external-base)
	       (code ,(or code
			  (and style `(or (page-style-code ,style) :ok))
			  :ok))
	       (meta ,(or meta
			  (and style `(page-style-meta ,style))))
	       (cookie ,(or cookie
			    (and style `(page-style-cookie ,style))))
	       (auth ,(or auth
			  (and style `(page-style-auth ,style))))
	       (background ,(or background
				(and style `(page-style-background ,style))))
	       (bgcolor ,(or bgcolor
			     (and style `(page-style-bgcolor ,style))))
	       (text ,(or text
			  (and style `(page-style-text ,style))))
	       (link ,(or link
			  (and style `(page-style-link ,style))))
	       (vlink ,(or vlink
			   (and style `(page-style-vlink ,style))))
	       (alink ,(or alink
			   (and style `(page-style-alink ,style))))
	       (*LispWeb-html-indent* 0))
	   (when auth
	     (check-auth auth))		;this does not return if auth check fails
	   (when (page-traced-p ',func-name)
	     (print-page-trace ',func-name))
	   (let ((the-page 
		  (if (service-loaded-p :hall)
		      (with-output-to-string (html-stream)
			(declare (special html-stream))
			(html-out "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">~%")
			(html-out "<HTML>~%<!-- Page ~a, produced by HALL, LispWeb ~a -->~%~%<HEAD>~%" 
				  ',func-name
				  *LispWeb-version*)
                        (if external-base
                            (html-out "<BASE HREF=~s>~%"
                                      external-base)
			(when true-base 
			  (html-out "<BASE HREF=~s>~%"
                                    (list-to-url (list *lispweb-server-hostname*
						       *http-port*
                                                       true-base)))))
			(html-out "<LINK REV=MADE HREF=\"mailto:~a\">~%" *operator-address*)
			
			
			(when meta 
			  (mapc #'(lambda (x)
				    (apply #'(lambda (&key http-equiv content)
					       (html-out "<META~:[~; HTTP-EQUIV=~:*~s~]~:[~; CONTENT=~:*~s~]>~%" 
							 http-equiv content))
					   x))
				meta))
			,(when javascript 
			   `(let ((jscript ,javascript))
			      (when (listp jscript)
				(setq jscript (apply #'concatenate 'string jscript)))
			      (html-out "~a~%" (javascript jscript ,javascript-source))))
			(html-out "<TITLE>~a</TITLE>~%" ,title)
			(html-out "</HEAD>~%~%")
			
			,(when frames
			   `(html-out "~a~%<NOFRAMES>~%" (apply #'frameset ',frames)))
			
			(html-out "<BODY~:[~; BACKGROUND=~:*~a~]~:[~; BGCOLOR=~:*~s~]~:[~; TEXT=~:*~s~]~:[~; LINK=~:*~s~]~:[~; VLINK=~:*~s~]~:[~; ALINK=~:*~s~]~:[~; OnLoad=~:*~a~]~:[~; OnUnload=~:*~a~]~:[~; LEFTMARGIN=~:*~a~]>~%" 
				  background bgcolor text link vlink alink ,onload ,onunload
                                  ,left-margin)
			,@forms
			(html-out "~&</BODY>~%")
			
			,(when frames 
			   '(html-out "</NOFRAMES>~%"))
			
			(html-out "</HTML>~%"))
		    "")))
             (when ,header-p
	       (http-header *http-text-header* :code-name code :length (length the-page)
			    :other-headers (when cookie (cookie-field cookie))))
	     (princ the-page *http-stream*)
	     t)))			;end of defun
	 
       ,(when url
	  `(register-page ,url ',func-name ',class))
       ',func-name)))

(defmacro define-no-page ((&key (func-name (gentemp "PAGE-"))
				(url (format nil "/~(~a~)" func-name))
				(class :user))
			  lambda-list &body forms)
  (let ((decl nil))
    (if (eq (caar forms) 'declare)	;prepare declarations
	(setq decl (append (car forms) (list *lispweb-declarations*))
	      forms (cdr forms))
      (setq decl (list 'declare *lispweb-declarations*)))
    (when (keyed-class-p class)
      (push 'http::lispweb-page-key lambda-list))  
    `(progn 
       (set-symbol-info ',func-name :page ,url ',class)
       (defun ,func-name 
	   ,lambda-list
	 ,decl
	 ,(when (keyed-class-p class)	
	    `(setq http::lispweb-page-key 
	       (test-access-key http::lispweb-page-key))) ;this does not return if key check fails
	 (when (page-traced-p ',func-name)
	   (print-page-trace ',func-name))
	 ,@forms)
       ,(when url
	  `(register-page ,url ',func-name ',class))
       ',func-name)))

#|
(defmacro http-force-output ()
  '(princ (get-output-stream-string html-stream) *http-stream*))
|#

(defmacro with-http-output (&body forms)
  `(with-output-to-string (html-stream)
     (declare (special html-stream))
     ,@forms))

(defmacro with-html-indent (&body forms)
  (let ((locvar (gensym)))
    `(let ((,locvar nil))
       (format t "Incrementing indent...~%")
       (incf *LispWeb-html-indent* 2)
       (setq ,locvar (progn ,@forms))
       (format t "Decrementing indent...~%")
       (decf *LispWeb-html-indent* 2)
       ,locvar)))
  
(defun add-lambda-list-keywords (lambda-list)
  (cond ((null lambda-list)
	 (cons '&key *lispweb-lambda-keywords*))
	((eq (car lambda-list) '&key)
	 (cons '&key (append *lispweb-lambda-keywords* (cdr lambda-list))))
	(t (cons (car lambda-list)
		 (add-lambda-list-keywords (cdr lambda-list))))))

(defmacro with-http-headers (headers &body forms)
  (let ((header-keys (mapcar #'(lambda (header-name)
				(intern (symbol-name header-name) :keyword))
			      headers)))
    `(let ((header-values (http::http-info-headers http::*http-info*)))
       (progv ',headers
	   (mapcar #'(lambda (header-key)
		       (getf header-values header-key))
		   ',header-keys)
	 ,@forms))))
  
(defmacro with-field-value (fields &body forms)
  "This macro should be used to access the values entered in a form. Fields is a list of
field names; during the execution of the forms, they are bound to the values of the 
corresponding form items."

  (let ((field-keys (mapcar #'(lambda (field-name)
				(intern (symbol-name field-name) :keyword))
			    fields)))
    `(let ,fields
       (declare (special ,@fields))
       (progv ',fields
	   (mapcar #'(lambda (field-key)
		       (form-value field-key))
		   ',field-keys)
	 ,@forms))))

(eval-when (load eval)
  (setf (symbol-function 'with-form-fields)
    (symbol-function 'with-field-value)))

(defmacro with-field-values (fields &body forms)
  "This macro should be used to access the values entered in a form. Fields is a list of
field names; during the execution of the forms, they are bound to the values of the 
corresponding form items."

  (let ((field-keys (mapcar #'(lambda (field-name)
				(intern (symbol-name field-name) :keyword))
			    fields)))
    `(progn 
       ;(declare (special ,@fields))
       (progv ',fields
	   (mapcar #'(lambda (field-key)
		       (form-values field-key))
		   ',field-keys)
	 ,@forms))))

(defmacro ordered-push (item list test)
  `(setf ,list
     (ordered-insert ,item ,list ,test)))

(defmacro define-dispatch (name list)
  `(progn
     (setf (get ',name :lispweb-fun-type) :dispatch)
     (defun ,name (request &optional arguments)
       (call-handler ,list request arguments))))

(defmacro define-application (name 
			      &key (class :user)
				   (url (format nil "/~(~a~)" name))
				   (pretty-name (format nil "~:(~a~)" name))
				   (link (anchor (concatenate 'string url "/top") pretty-name))
				   (key-p nil))
  "This macro defines an application, that is a set of related pages.
name        - the name of the application
class       - the class to which the application belongs (:user by default)
url         - the url of this application 
pretty-name - the full name of the application
link        - an anchor to the top page of the application; if not nil, it will be
              inserted in the top-level links
key-p       - if T, all pages in this application are protected by keys."
  
  `(progn 
     (defvar ,name nil)			;define the list variable
     ;(define-dispatch ,name ,name)	;define the dispatch function
     (register-page ,url ',name ',class);register it in user pages
     (set-symbol-info ',name :dispatch ,url ',class)
     ,(when link
	`(register-top-level-link ,link))
     ,(when key-p
	`(pushnew ',name *keyed-classes*))))

(defmacro region-cond (x y &rest specs)
  "Use this macro to handle static image-maps. X and Y are the coordinates of 
the mouse click; the specs are lists of the form (region-specifier &rest forms). 
A region specifier is a list with one of the following forms:
(:rect x1 y1 x2 y2) - rectangular region
(:circle xc yc r) - circular region
(:default)        - default region (this should be the last spec)."
  
  (let ((body (mapcar #'(lambda (spec)
			  `((point-in-area-p ,x ,y ',(car spec))
			    ,@(cdr spec)))
		      specs)))
    (cons 'cond body)))

(defmacro defun-hall (name args &body body)
  `(progn 
     (setf (get ',name :lispweb-arglist) ',args)
     (defun ,name ,args
       ,@body)))

;;; This should be moved somewhere else
(defun hall-arglist (name)
  (get name :lispweb-arglist))

(defmacro with-file-lines ((line-name filename &rest open-options) &body forms)
  "This macro opens the specified file and executes the forms once for every line,
binding line-name to the contents of each line in turn."
  
  (let ((stream-name (gensym)))
    `(with-open-file (,stream-name ,filename ,@open-options)
       (let ((,line-name nil))
	 (loop
	   (unless (setq ,line-name (read-line ,stream-name nil nil))
	     (return))
	   ,@forms)))))

(defmacro add-lispweb-service (&rest args)
  `(eval-when (load eval)
     (add-service ,@args)))

;;; Platform independent macro to time a form

(defmacro time-form (&body form)
  (let ((t1 (gensym))
	(t2 (gensym)))
    `(let ((,t1 (get-internal-real-time)))
       ,@form
       (let ((,t2 (get-internal-real-time)))
	 (/ (- ,t2 ,t1)
	    internal-time-units-per-second)))))

;;; Reader macro to concatenate strings easily:
;;; #{"one, " "two, " "three."} -> "one, two, three."

(eval-when (load eval compile)
  (defun open-cbrace-macro-char (stream macro-char dummy)
    (declare (ignore macro-char dummy))
    (let ((data (read-delimited-list #\} stream t)))
      `(concatenate 'string ,@data))))

(eval-when (load eval compile)
  (set-dispatch-macro-character #\# #\{ #'open-cbrace-macro-char))

#| Old code...

(defmacro html-out (format-string &rest arguments)
  `(princ (apply (formatter ,format-string) nil ,@arguments) html-stream))
  
(defun constant-string-p (item)
  (or (stringp item)
      (numberp item)
      (keywordp item)
      (and (listp item)
	   (or (eq (car item) 'quote)
	       (and (get (car item) :lispweb-arglist)
		    (every #'constant-string-p (cdr item)))))))

(defmacro define-page (&whole definition
			      (title &key (func-name (gentemp "PAGE-"))
					  (url (format nil "/~(~a~)" func-name))
					  (code :ok)
					  meta
					  frames
					  javascript
					  (package :http)
					  background
					  base
					  bgcolor
					  text
					  link
					  vlink
					  alink
					  onload
					  onunload
					  auth
					  (class :user)
					  style
					  (visible *LispWeb-visible-pages*))
			      lambda-list &body forms)
  "This macro defines a function to build a WWW page and registers it.
title       - a string that names the page
func-name   - the name of the defined function
url         - the url of the function 
code        - the http response code associated with the page
meta        - list of meta fields in header
frames      - a frameset specifier
javascript  - JavaScript code to be included in the document's head
package     - the package in which the function should run
background  - background image
base        - base href
bgcolor,
text,
link,
vlink,
alink       - colors
auth        - authorization list for the page
class       - class of the page, :user, :system, or a class name
style       - a page-style structure
visible     - flag for source recording
lambda-list - parameters of the defined function
forms       - body of the function"
  
  (let ((decl nil))
    (if (eq (caar forms) 'declare)	;prepare declarations
	(setq decl (append (car forms) (list *lispweb-declarations*))
	      forms (cdr forms))
      (setq decl (list 'declare *lispweb-declarations*)))
;    (setq lambda-list			;prepare lambda-list
;     (add-lambda-list-keywords lambda-list))
    (when visible
      (setq forms (append forms (list `(html-out (source-page-button ',func-name))))))
    `(let ((true-base nil))
       (set-symbol-info ',func-name :page ,url ',class)
       (setq true-base (or ,base		;use supplied BASE if present
			   (when ,url (url ',func-name)))) ;or page address if present
       ,(when visible
	  `(setf (get ',func-name :hall-source) ',definition))
       (defun ,func-name ,lambda-list
	 ,decl
	 ,(when auth
	    `(check-auth ,auth))	;this does not return if auth check fails
	 (with-output-to-http ,title ,code ,meta ,frames ,javascript ,background true-base
			      ,bgcolor ,text ,link ,vlink ,alink ,style ,onload ,onunload
			      (let ((*package* (find-package ,package)))
				,@forms)))
       ,(when url
	  `(register-page ,url ',func-name ',class))
       ',func-name)))

(defmacro with-output-to-http (title 
			       code 
			       meta
			       frames
			       javascript
			       background
			       base
			       bgcolor
			       text
			       link 
			       vlink
			       alink
			       style
			       onload 
			       onunload
			       &body forms)
  `(let ((code ,(or code (and style `(page-style-code ,style))))
	 (meta ,(or meta (and style `(page-style-meta ,style))))
	 (background ,(or background (and style `(page-style-background ,style))))
	 (bgcolor ,(or bgcolor (and style `(page-style-bgcolor ,style))))
	 (text ,(or text (and style `(page-style-text ,style))))
	 (link ,(or link (and style `(page-style-link ,style))))
	 (vlink ,(or vlink (and style `(page-style-vlink ,style))))
	 (alink ,(or alink (and style `(page-style-alink ,style))))
	 (frameset ',(car frames))
	 (framelist ',(cdr frames))
	 (jscript ,javascript)
	 (onload ,onload)
	 (onunload ,onunload))
     (let ((the-page 
	    (if (service-loaded-p :hall)
		(with-output-to-string (html-stream)
		  (declare (special html-stream))
					;(html-out "<!DOCTYPE HTML PUBLIC \"~~//IETF//DTD HTML 2.0\\EN\">~%")
		  (html-out "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">~%")
		  (html-out "<HTML>~%<!-- Page produced by HALL, LispWeb 1.0 -->~%~%<HEAD>~%")
		  (when ,base (html-out "<BASE HREF=~s>~%" (list-to-url (list *lispweb-server-hostname*
									      *http-port*
									      ,base))))
		  (html-out "<LINK REV=MADE HREF=\"mailto:~a\">~%" *operator-address*)
		  (mapc #'(lambda (x)
			    (apply #'(lambda (&key http-equiv content url)
				       (html-out "<META~:[~; HTTP-EQUIV=~:*~a~]~:[~; CONTENT=~:*~a~]~:[~; URL=~:*~a~]>" http-equiv content url))
				   x))
			meta)
		  (format t "jscript=~a~%" jscript)
		  (when jscript 
		    (when (listp jscript)
		      (setq jscript (apply #'concatenate 'string jscript)))
		    (html-out (javascript jscript)))
		  (html-out "<TITLE>~a</TITLE>~%" ,title)
		  (html-out "</HEAD>~%~%")
		  (when frameset
		    (html-out (apply #'frameset frameset))
		    (mapc #'(lambda (fr)
			      (declare (ignorable fr))
			      (html-out (apply #'frame fr)))
			  framelist)
		    (html-out "</FRAMESET>~%<NOFRAMES>~%"))
		  (html-out "<BODY~:[~; BACKGROUND=~:*~a~]~:[~; BGCOLOR=~:*~s~]~:[~; TEXT=~:*~s~]~:[~; LINK=~:*~s~]~:[~; VLINK=~:*~s~]~:[~; ALINK=~:*~s~]~:[~; OnLoad=~:*~a~]~:[~; OnUnload=~:*~a~]>~%" 
			    background bgcolor text link vlink alink onload onunload)
		  ,@forms
		  (html-out "</BODY>~%")
		  (when frameset (html-out "<NOFRAMES>~%"))
		  (html-out "</HTML>~%"))
	      "")))
       (http-header *http-text-header* :code-name code :length (length the-page))
       (princ the-page *http-stream*)
       t)))

|#