(in-package :tcp)

;;; Interface functions

(defun close-tcp-stream (stream)
  "Closes the stream's socket."
  (close (two-way-stream-input-stream stream))
  (close (two-way-stream-output-stream stream))
  (close stream))

(defun get-host-by-addr (address)
  (handler-case
      #+lispworks3.2.2
    (comm::get-host-name-by-address (comm::ip-address-string-to-number address))
    #+lispworks4
    (if *unplugged-p*
        "localhost"
        (comm::get-host-entry address :fields '(:name)))
    (error (c)
           ;;(declare (ignore c))
           address)))

#| for pc version (foreign:define-foreign-function 
 (crypt "_crypt")
 ((key :string) (salt :string))
 :result-type :string)
|#

#-:lispworks(defun full-hostname ()
  (let ((name (with-output-to-string (x)
                (system::call-system-showing-output "hostname" 
                                                    :prefix "" 
                                                    :output-stream x 
                                                    :show-cmd nil)))
        (domain (with-output-to-string (x)
                  (system::call-system-showing-output "domainname" 
                                                      :prefix "" 
                                                      :output-stream x 
                                                      :show-cmd nil))))
    (format nil "~a.~a" (subseq name 0 (1- (length name))) 
                       (subseq domain 0 (1- (length domain))))))

#+:lispworks
(defun full-hostname ()
  (if *unplugged-p*
      tcp::*host-ip-address*
    (http::machine-address)))

#|
(defun socket-fd (stream)
  "Return the FD associated with a socket stream"
  (excl::stream-input-fn stream))

;;; Support for external C functions

(load (merge-pathnames "hostadd.o" *load-pathname*)
      :foreign-files (list (merge-pathnames "libres.a" *load-pathname*))
      :unreferenced-lib-names '("_res_gethostbyaddr" "_crypt"))

(ff:defforeign 'res-hostadd 
    :entry-point "_res_hostadd" 
    :arguments '(string) 
    :return-type :integer)

(ff:defforeign 'res-hostname
    :entry-point "_res_hostname"
    :arguments '(string)
    :return-type :integer)

(defun get-host-by-name (name)
  (char*-to-string (res-hostname name)))

(ff::defforeign 'c-crypt
    :entry-point "_crypt"
    :arguments '(string string)
    :return-type :integer)

(defun crypt (key salt)
  (ff::char*-to-string (c-crypt key salt)))

(ff:defforeign 'c-full-hostname
    :entry-point "_full_hostname"
    :return-type :integer)

(defun full-hostname ()
  (ff:char*-to-string (c-full-hostname)))
|#

(defun get-host-by-name (name)
  name)

;(defun full-hostname ()
;  (system::getenv "HOST"))

;;; Two simple HTTP clients.

(defun telnet (host port)
  (comm::open-tcp-stream host port))

(defun send-mail (sender recipient text)
  "Send the mail message contained in text to recipient, using sender as the sender name."
  (let* ((host (subseq recipient (1+ (position #\@ recipient))))
	 (s (telnet host 25)))
    (when (= (read-from-string (read-line s)) 220)
      (format s "HELO~%")
      (force-output s)
      (format s "MAIL From:<~a>~%" sender)
      (force-output s)
      (format s "RCPT To:<~a>~%" recipient)
      (force-output s)
      (format s "DATA~%")
      (force-output s)
      (format s "~A~%.~%" text)
      (force-output s)
      (format s "QUIT~%")
      (force-output s))
    (close s)))

(defun lisp-http-server ()
  (setq http::*LispWeb-server-hostname* (full-hostname))
  (http::add-lispweb-process
   :http
   (comm::start-up-server
    :function 'http-server-startup-stream
    :service http::*http-port*
    :process-name (format nil "LispWeb server port ~a" http::*http-port*)))
  (http::http-log-message "Lispweb server started on port ~a" http::*http-port*))


(defun lisp-http-server-with-mp ()
  (setq http::*LispWeb-server-hostname* (full-hostname))
  (http::add-lispweb-process
   :http
   (comm::start-up-server-and-mp
    :function 'http-server-startup-stream
    :service http::*http-port*
    :process-name (format nil "LispWeb server port ~a" http::*http-port*)))
  (http::http-log-message "Lispweb server started on port ~a" http::*http-port*))


(defun http-server-startup-stream (fd &aux stream integer-network-address addr)
  (let (istream ostream 
                (*trace-output* *standard-output*))
    (multiple-value-setq (istream ostream)
        (comm::streams-from-fd fd))
    (setq stream (make-two-way-stream istream ostream))
    #+harlequin-pc-lisp
    (setq integer-network-address (comm::socket-stream-peer-address istream))
    #+lispworks4.1
    (setq integer-network-address (comm::socket-stream-peer-address istream))
    #+lispworks3.2.2
    (setq integer-network-address (comm::get-peer-host-and-port fd))
    #+harlequin-pc-lisp
    (setq addr (comm::socket-stream-peer-address istream))
    #+harlequin-unix-lisp
    (setq addr (comm::ip-address-string integer-network-address))
    (let ((info (http::make-http-info :socket http::*http-port*
				      :stream stream
				      :fd fd
				      :addr addr
				      :request-counter (incf http::*request-counter*))))
      (http::lw-log :debug ";;; Connection received from ~a~%" addr)
      (let ((proc (mp:process-run-function (format nil "Http Handler #~d" http::*request-counter*)
                                           nil
                                           'handle-request
                                           info)))
        (http::add-lispweb-process http::*request-counter* proc)
        (http::lw-log :debug ";;; Process ~d started~%" http::*request-counter*)))))

(defun handle-request (info)
  (let ((http::*http-info* info)
	(stream (http::http-info-stream info)))
    (unwind-protect
	(http::process-request http::*http-info*)
      #|
      (unless (equalp (http::header-value :connection) "Keep-Alive")
	(return))
      (unless (wait-for-input-available stream
					:whostate "Waiting for next request"
					:timeout 10)
	(return))
      (http::lw-log :debug ";;; Keeping connection open~%"))
      |#
      (handler-case
      (progn
	(http::lw-log :debug ";;; CLosing connection~%")
	(close-tcp-stream stream) ;close stream at the end
	(http::remove-lispweb-process
         (http::http-info-request-counter http::*http-info*)))
        (stream-error (c) ;connection interrupted
                      (declare (ignore c))
                      (format t  "Connection interrupted in handle request"))
                (file-error (c)	;connection interrupted
			    (declare (ignore c))
                            (format t  "File error in handle request")
			    nil)
                (error (c)
                       (format t
                               "Connection interrupted - reason unknown. In handle request")))))) ; and remove process

(defun wait-for-input-available (stream &key whostate timeout)
  "Returns T if timedout, NIL if input available."
  (mp:process-wait-with-timeout whostate
                                timeout
                                #'listen
                                stream))

(defun start-telnet-server ()
  (http::add-lispweb-process :telnet
    (comm::start-up-server :function 'listener-top-level
		           :service http::*eval-port*
                           :process-name (format nil "Telnet server port ~a" http::*eval-port*)))
  (http::http-log-message "Telnet server started on port ~a" http::*eval-port*))

(defun start-telnet-server-with-mp ()
  (http::add-lispweb-process
   :telnet
   (comm::start-up-server-and-mp
    :function 'listener-top-level
    :service http::*eval-port*
    :process-name (format nil "Telnet server port ~a" http::*eval-port*)))
  (http::http-log-message "Telnet server started on port ~a" http::*eval-port*))

#|
(defun start-telnet-server ()
  (let (inet-port)
    (http::add-lispweb-process :telnet
      (start-lisp-server-process 
       :server-name "Telnet Server"
       :inet-port http:*eval-port*
       :startup-stream-process #'(lambda (stm &rest args)
				   (declare (ignore args))
				   (let ((client-address
					  (getf (excl::stream-property-list stm) :pretty-socket-id)))
				     (format t "~&;Accepting telnet connection ~a/~a~%" client-address inet-port)
				     (mp:process-run-function
				      (format nil "Telnet Client~@[ ~a~]" client-address)
				      #'listener-top-level
				      stm)))
       :server-startup-announce-function #'(lambda ()
					     (setq inet-port *inet-port-used*)
					     (setf (mp:process-name mp:*current-process*)
					       (format nil "Telnet Server port ~a" inet-port))
					     (http::http-log-message "Telnet server started on port ~a" http:*eval-port*))))))
|#

(defvar *banner-string* "~a ~a~%Copyright The Harlequin Group Limited 1987--1995~%")

(defun listener-top-level (fd &aux stream)
  (multiple-value-bind (istream ostream)
      (comm::streams-from-fd fd)
    (setq stream (make-two-way-stream istream ostream)))
  (format stream *banner-string* (lisp-implementation-type) (lisp-implementation-version))
  (format stream "~&~%Enter password: ")
  (force-output stream)
  (let ((pass (remove #\Return (read-line stream))))
    (when (http:check-password pass)
            ;(unread-char #\Newline stream)
      (ignore-errors (comm::make-stream-and-run-listener fd)))))

