(in-package :http)

(defstruct service
  name
  description
  (version 1)
  (revision 0)
  (anchor 'no-service-doc)
  revision-date)

(defun print-service (service)
  (format nil "~a, version ~d.~d (~a)" 
	  (service-name service)
	  (service-version service)
	  (service-revision service)
	  (service-revision-date service)))

(defun service-link (service)
  (page-anchor (service-anchor service) (print-service service) :args (list (service-name service))))

(defun create-service (name 
                       description
                       version
                       anchor)
  (make-service :name name
                :description description
                :version version
                :anchor anchor
                :revision-date (http-time)))

(defun add-service (name &key description
                              (version 1)
                              anchor)
  (let ((serv (service-loaded-p name)))
    (cond (serv 
           (when description
             (setf (service-description serv) description))
           (when anchor
             (setf (service-anchor serv) anchor))
           (setf (service-revision-date serv)
             (http-time))
           (if (and version
                    (not (= version (service-version serv))))
              (setf (service-version serv) version
                    (service-revision serv) 0)
              (incf (service-revision serv))))
        (t (push (create-service name
                                 description
                                 version
                                 anchor)
                 *lispweb-services*)))))

(defun service-loaded-p (name)
  (find name *LispWeb-services* :key #'service-name))

