(in-package :http)

;;; Predefined error pages

(define-page ("Page not found!" :func-name page-not-found :url nil :class :system :code :not-found)
    (request)
  (html-out *LispWeb-banner*)
  (html-out "The requested page, ~a, was not found on this server. Sorry :(~%" request)
  (html-out (header 4 (format nil "<HR>Go back to the ~a~%" (anchor "/" "top page")))))

(define-page ("Form not found!" :func-name form-not-found :url nil :class :system :code :not-found)
    (request)
  (html-out *LispWeb-banner*)
  (html-out "This server is unable to execute the ~s action. Sorry :(~%" request)
  (html-out (header 4 (format nil "<HR>Go back to the ~a~%" (anchor "/" "top page")))))

(define-page ("Map not found!" :func-name map-not-found :url nil :class :system :code :not-found)
    (map)
  (html-out *LispWeb-banner*)
  (html-out "The ~s map is unknown on this server. Sorry :(~%" map)
  (html-out "<HR><H4>Go back to the ~a</H4>~%" (anchor "/" "top page")))

(define-page ("Internal Error!" :func-name error-page :url nil :class :system :code :error) 
    (condition)
  (html-out *LispWeb-banner*)
  (if (member (type-of condition) '(simple-error simple-type-error simple-warning simple-condition))
      (html-out "Your request caused the following error: ~%~a" 
		(plaintext (apply #'format nil #+LUCID (simple-condition-format-string condition)
				  #+ALLEGRO (simple-condition-format-control condition)
                                  #-lispworks (list (format nil "~a"
                                                            (simple-condition-format-arguments
                                                             condition)))
                                  
                                  #+lispworks  (system::simple-condition-format-string
                                                             condition)
                                  #+lispworks (simple-condition-format-arguments
                                                             condition)
				  #-lispworks (simple-condition-format-arguments condition)
                                  )))
    (html-out "Your request caused an internal error of type ~a in the server.~%" 
	      (bold (type-of condition))))
  (html-out "Debug information:~%<P>")
  (format-http-info)
  (html-out "<P>The server should be able to recover, please inform the ~a in case this does not happen." 
	    (anchor "mailto:alb@ipvaim.unipv.it" "author"))
  (html-out "<HR><H4>Go back to the ~a</H4>~%" (anchor "/" "top page")))

(define-page ("Syntax error!" :func-name bad-syntax-page :url nil :class :system :code :bad-syntax)
    (request)
  (html-out *LispWeb-banner*)
  (html-out "The syntax of your request (~s) is incorrect.~%" request)
  (html-out "<HR><H4>Go back to the ~a</H4>~%" (anchor "/" "top page")))
  
(define-page ("No Authorization!" :func-name no-auth-page :url nil :class :system :code :forbidden)
    (request)
  (html-out *LispWeb-banner*)
  (html-out "You are not authorized to access the ~s page.~%" request)
  (html-out "<HR><H4>Go back to the ~a</H4>~%" (anchor "/" "top page")))
  
(define-page ("No Authorization!" :func-name bad-key-page :url nil :class :system :code :forbidden)
    (request)
  (html-out *LispWeb-banner*)
  (html-out "You don't have the access key for the ~s page.~%" request)
  (html-out "<HR><H4>Go back to the ~a</H4>~%" (anchor "/" "top page")))
  
(define-page ("No Authorization!" :func-name key-expired-page :url nil :class :system :code :forbidden)
    (request)
  (html-out *LispWeb-banner*)
  (html-out "Your access key for the ~s page has expired.~%" request)
  (html-out "<HR><H4>Go back to the ~a</H4>~%" (anchor "/" "top page")))
  
(define-page ("Service Documentation" :func-name no-service-doc :class :system)
    (serv-name)
  (html-out *LispWeb-banner*)
  (html-out "Sorry, no documentation is available for the ~a service." (bold serv-name)))
