;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology wsmo-use-case-train-connection)


;; location, border from loc
(def-class station (location) ?s
  ((has-code :type string
             :max-cardinality 1)
   (has-locatedIn :type location)
   (has-borderToCountry :type border
                        :max-cardinality 1))
; constraint "<-
;                   S[
;                         locatedIn hasvalue L,
;                         country hasvalue C]
;                   and not L[
;                         country hasvalue C]."
  :constraint (and (has-locatedIn ?s ?s-location)
                   (has-country ?s-location ?s-l-country)
                   (has-country ?s ?s-country)
                   (= ?s-country ?s-l-country)))

;; location from loc
(def-class itinerary ()
  ((has-passenger :type person
                  :max-cardinality 1)
   (has-recordLocatorNumber :type string
                            :max-cardinality 1)
   (has-trip :type trip
             :max-cardinality 1)))

;; location, distance from loc; interval,dateAndTime,before from date-time
(def-class trip () ?t
  ((has-start :type location
              :max-cardinality 1)
   (has-end :type location
            :max-cardinality 1)
   (has-via :type location)
   (has-departure :type dateAndTime
                  :max-cardinality 1)
   (has-arrival :type dateAndTime
                :max-cardinality 1)
   (has-duration :type interval
                 :max-cardinality 1)
   (has-distance :type distance 
                 :max-cardinality 1))
  ;;constraint "<-
;                   T[
;                         departure hasvalue D,
;                         arrival hasvalue A]
;                   and A <= D."
  :constraint (and (has-arrival ?t ?t-arrival)
                   (has-departure ?t ?t-departure)
                   (not (before ?t-arrival ?t-departure)
                        (has-start ?t ?t-start)
                        (has-end ?t-end)
                        (not (equals-to ?t-start ?t-end)))))

(def-class trainTrip (trip)   
  ((has-start :type station
              :max-cardinality 1)
   (has-end :type station
            :max-cardinality 1)
   (has-via :type station)
   (has-seat :type string
             :max-cardinality 1)
   (has-train :type string
              :max-cardinality 1)
   (has-class :type string
              :max-cardinality 1)))

;;comment: instanceDefinitions

;; innsbruck from loc
(def-instance innsbruckHbf station
((has-name "Innsbruck Hbf")
 (has-code "INN")
 (has-locatedIn innsbruck)))

;; frankfurt from loc
(def-instance frankfurtHbf station
((has-name "Frankfurt Hbf")
 (has-code "FKF")
 (has-locatedIn frankfurt)))
