;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology akt-kmi-planet-psm-ontology)


(def-class semantic-search-psm
           (primitive-method)
           ?psm
           ((has-input-role :value search-item)
            (has-output-role :value stories)
            (search-item :type thing)
            (stories :type list)
            (has-precondition
             :value
             (kappa (?task) (thing (role-value ?task search-item))))))




(def-class event-semantic-search-service1 (semantic-search-psm) ?psm
  ()
  :own-slots ((tackles-task-type semantic-story-search-task)
              (applicability-condition 
               (kappa (?task)
                      (event (role-value ?task search-item))))))

(def-class event-type-semantic-search-service (semantic-search-psm) ?psm
  ()
  :own-slots ((tackles-task-type semantic-story-search-task)
              (applicability-condition 
               (kappa (?task)
                      (event-type (role-value ?task search-item))))))

(def-class person-semantic-search-service
           (semantic-search-psm)
           ?psm
           ((has-precondition
             :value
             (kappa (?task) (thing (role-value ?task search-item)))))
           :own-slots
           ((applicability-condition
             (kappa (?task) (person (role-value ?task search-item))))
            (tackles-task-type semantic-story-search-task)))

(def-class person-type-semantic-search-service
           (semantic-search-psm)
           ?psm
           ((has-precondition
             :value
             (kappa
              (?task)
              (or (thing (role-value ?task search-item))
                  (thing-type (role-value ?task search-item))))))
           :own-slots
           ((applicability-condition
             (kappa
              (?task)
              (person-type (role-value ?task search-item))))
            (tackles-task-type semantic-story-search-task)))


(def-class personalised-stories-psm  (primitive-method) ?psm
  ((has-input-role :value has-profile-name
                   :value has-personal-profile-item)
   (has-output-role :value planet-page)
   (has-profile-name :type symbol)
   (has-personal-profile-item :type thing)
   (planet-page :type url)
   (has-precondition 
    :value (kappa (?task)
                  (thing (role-value ?task has-personal-profile-item)))) 
   (has-goal-expression 
    :value (kappa (?task ?sol)
                  (= ?sol (related-kmi-planet-stories 
                           (role-value ?task 
                                       has-personal-profile-item))))))
   :own-slots
   ((tackles-task-type personalised-stories-task)))


(def-class related-items-psm (primitive-method) ?psm
  ((has-input-role :value  has-profile-name
                   :value has-personal-profile-item)
   (has-output-role :value profile)
   (has-profile-name :type symbol)
   (has-personal-profile-item :type thing)
   ;;the head of the list is the profile name the rest 
   ;;are the related items
   (profile :type list))
  :own-slots
  ((tackles-task-type related-items-task)))


;;;takes a set of related items and returns the url 
;;;of a personalised planets
(def-class make-personalised-planet-psm (primitive-method) ?psm
  ((has-input-role :value  has-profile-name
                   :value has-personal-profile-item)
   (has-output-role :value planet-page)
   (has-profile-name :type symbol)
   (has-personal-profile-item :type thing)
   (planet-page :type url))
  :own-slots
  ((tackles-task-type make-personalised-planet-task)))

(def-class add-1-psm
           (primitive-method)
           ?psm
           ((has-input-role :value input-number)
            (has-output-role :value output-number)
            (input-number :type number)
            (output-number :type number))
           :own-slots
           ((tackles-task-type add-1-task)))

(def-class  multiply-by-2-psm  (primitive-method) ?psm
  ((has-input-role :value input-number)
   (has-output-role :value output-number)
   (input-number :type number)
   (output-number :type number))
  :own-slots
   ((tackles-task-type multiply-by-2-task)))


(def-class add-1-task-to-multiply-by-2-task-mapping (primitive-method)
  "Maps from add-1-task to multiply-by-2-task"
  ((has-input-role :value output-task
                   :value input-task
                   :value output-task-output)
   (has-output-role :value input-task-input)
   (output-task :type goal-specification-task)
   (input-task :type goal-specification-task)
   (output-task-output :type ocml-expression)
   (input-task-input :type list))
  :own-slots
  ((tackles-task-type task-io-mapping-task)
   (applicability-condition 
               (kappa (?task)
                      (and (= 'add-1-task
                            (role-value ?task output-task))
                           (= 'multiply-by-2-task
                            (role-value ?task input-task)))))))



(def-class return-rdf-psm (primitive-method) ?psm
  ((has-input-role :value input1)
   (has-output-role :value rdf-output)
   (input1 :type thing)
   (rdf-output :type string))
  :own-slots
  ((tackles-task-type return-rdf-task)))
