;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology climate-prediction-services)

(def-rule climate-prediction-services-rule
  ((has-services ?item (meteorology-category ?item
                        explain find-examples relevant-news-stories
                        relevant-discussions related-concepts 
                        relationship-to-model extreme-cases 
                        related-people-or-groups)) 
   ;;find examples
   ;;relevant news stories
   ;;relevant discussions
   ;;related concepts
   ;;relationship to model
   ;;extreme cases
   ;;related people/groups
   if
   (is-in-category ?item ?category)
   (meteorology-category-type ?category))
  ((has-services ?item (climatology-category ?item
                        explain show-in-model relevant-news-stories 
                        relevant-discussions related-concepts 
                        related-people-or-groups)) 
   if
   (is-in-category ?item ?category)
   (climatology-category-type ?category))
  ((has-services ?item (physics-category ?item
                        explain show-visualization)) 
   if
   (is-in-category ?item ?category)
   (physics-category-type ?category))
  ((has-services ?item (maths-category ?item
                        explain)) 
   if
   (is-in-category ?item ?category)
   (maths-category-type ?category))
  ((has-services ?item (working-person ?item
                                       person-details works-with 
                                       shares-interests-with 
                                       related-stories invite-to
                                       current-projects 
                                       projects-might-be-interested-in
                                       web-page))
   if 
   (working-person ?item))                
  ((has-services ?item (unknown ?item))))


(def-rule climate-prediction-explain-rule
    ((explain ?glossary-item ?text-definition ?image-base ?image-url ?image-label)
     if
     (has-definition ?glossary-item ?definition)
     (has-text-definition ?definition ?text-definition)
     (has-image ?definition ?image-definition)
     (has-base ?image-definition ?image-base)
     (has-URL ?image-definition ?image-url)
     (has-pretty-name ?image-definition ?image-label)
     )
  ((explain ?glossary-item ?text-definition nil nil nil)
     if
     (has-definition ?glossary-item ?definition)
     (has-text-definition ?definition ?text-definition)
     ))

;; Martin -- define new rule for explain with image (if exists)

;(def-rule climate-prediction-explain-rule
;    ((explain ?glossary-item ?text-definition ?image-base ?image-url ?image-name)
;     if
;     (has-definition ?glossary-item ?definition)
;     (has-text-definition ?definition ?text-definition)
;     (has-image ?definition ?image-definition)
;     (has-base ?image-definition ?image-base)
;     (has-URL ?image-definition ?image-url)
;     (has-pretty-name ?image-definition ?image-label)))

;; Martin -- define new rule for retrieving related parts of the course material
(def-rule climate-prediction-find-in-course-rule
  ((find-in-course ?glossary-item ?base-url ?link-url ?link-name)
   if
   (has-definition ?glossary-item ?definition)
   (has-link-to ?definition ?link)
   (url-ou-document-link ?link)
   (has-base ?link ?base-url)
   (has-URL ?link ?link-url)
   (has-pretty-name ?link ?link-name)))

(def-rule climate-prediction-find-in-encyc-rule
  ((find-in-encyc ?glossary-item ?base-url ?link-url ?link-name)
   if
   (has-definition ?glossary-item ?definition)
   (has-link-to ?definition ?link)
   (url-ext-document-link ?link)
   (has-base ?link ?base-url)
   (has-URL ?link ?link-url)
   (has-pretty-name ?link ?link-name)))

(def-rule climate-prediction-related-people-or-groups-rule
  ((related-people-or-groups 
    ?item 
    ;;"http://localhost/cp-pages/ou-earth-sciences.htm"
    "http://localhost/cp-pages/met-reading.htm"
    )
   if
   (is-in-category ?item ?category)
   (meteorology-category-type ?category))
   ((related-people-or-groups 
    ?item 
    "http://localhost/cp-pages/aopp-oxford.htm") 
   if
   (is-in-category ?item ?category)
   (climatology-category-type ?category)))
