;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-



(in-package "OCML")

;;KL View
(def-class design-state ()
  ((has-design-model :type design-model :cardinality 1)
   
   ))

;;;Symbol Level View
(defclass  design-state ()
  ((state-operators :initform nil :accessor state-operators)
   (complete? :initform nil :accessor state-complete?)
   (feasible? :initform nil :accessor state-feasible?)
   (failed? :initform nil :accessor state-failed?)
   (new-state? :initform t :accessor new-state?)
   (depth  :accessor state-depth :initarg :depth :initform 0)
   (cost  :accessor state-cost  :initform 0)
   (state-foci :initform nil :accessor state-foci)
   (current-focus :initform nil :accessor current-focus)
   (current-operator :accessor current-operator)
   (nogood-values :initform nil :accessor nogood-values)
   (focus-operators :initform nil :accessor focus-operators)
   (violated-constraints :initform nil :accessor violated-constraints)
   (bad-assignments :initform nil 
                    :accessor bad-assignments
                    :documentation "An alist ((c . (p . v))...), where c is a constraint
                                    violation and (p. v) the assignment which violates it")
   (predecessor :initarg :predecessor :accessor predecessor :initform nil)
   ;;(continuation :initarg :continuation :accessor state-continuation :initform nil)
   (design-model :initform nil :initarg :design-model :accessor design-model)
   (design-removals :initarg :design-removals 
                    :initform nil
                    :accessor design-removals)
   (design-extensions :initarg :design-extensions 
                      :initform nil
                      :accessor design-extensions)))

(def-function COLLECT-CONSTRAINT-VIOLATIONS (?s ?task)
  "Takes a state and a parametric design task and returns
   the set of task constraints which are violated by the
   design model associated with the state"
  :constraint (and (design-state ?s)
                   (parametric-design ?task))
  :body (setofall ?c (and (has-design-model ?s ?d)
                          (member ?c (the ?l (has-constraints ?task ?l)))
                          (design-model-violates ?d ?c))))

(def-function COMPUTE-STATE-COST (?s ?task)
  :constraint (and (design-state ?s)
                   (parametric-design ?task))
  :body (call (the ?f (has-cost-function ?task ?f))
              (the ?d (has-design-model ?s ?d))))

(def-function COMPUTE-OPERATOR-COST (?op ?task)
  :constraint (and (design-operator ?op)
                   (parametric-design ?task))
  :body (if (and (has-cost-difference-function ?task ?fun)
                 (state-transition ?s1 ?op ?s2))
          (call ?fun ?s2 ?s1)))
  


(def-relation STATE-TRANSITION (?s1 ?design-op ?s2)
  :iff-def (and (design-state ?s1 has-design-model ?d1)
                (design-state ?s2 has-design-model ?d2)
                (design-operator ?design-op body ?fun)
                (= ?d2 (call ?fun ?d1))
                (not (= ?d1 ?d2))))


(def-function PREDECESSOR (?s)
  :constraint (design-state ?s)
  :body (the ?s1 (state-transition ?s1 ?op ?s)))

(def-function SUCCESSOR (?s)
  :constraint (design-state ?s)
  :body (the ?s1 (state-transition ?s ?op ?s1)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmethod print-object ((state design-state) stream)
  (with-slots (depth design-extensions cost design-model)state
    (format stream "#<STATE ~{~S ~}, dpth=~s, ext=~s, cst=~s>" design-extensions depth 
            (length design-model) cost)))

