;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-

(in-package "OCML")



(in-ontology propose-and-improve)

(defvar *branching-factor-per-parameter* 3)
(defvar *cost-improvement-measure* 0.2
  "we do not consider any improvement below this threshold")

(def-class PROPOSE&IMPROVE (design-method)
   :lisp-slots ((design-state-type :initform 'p&i-design-state)
               )
   :lisp-class-name propose&improve)

(defclass P&I-DESIGN-STATE (design-state)
  ((detailed-cost :accessor state-detailed-cost :initform nil)))

(defmethod state-fully-expanded ((state P&I-DESIGN-STATE))
  ;;;;(when *trace-design-process?*
    (print-with-spaces 2 ;;; (state-depth state) 
                       "State ~s fully expanded" state)
  ;;;;(setf (state-failed?  state) t)
  ;;(remove-design-state state)
  nil)

(def-class multiple-IMPROVE-OPERATOR (multiple-design-modification-operator)
  :lisp-class-name multiple-improve-operator)

(def-class BASIC-IMPROVE-OPERATOR (basic-design-modification-operator)
  :lisp-class-name basic-improve-operator)

;(defmethod DESIGN-FROM-STATE (appl (method propose&improve) (state p&i-design-state))
;  (if (state-feasible? state)
;    (if (violated-constraints state)
;      (state-fails state)
;      (if (state-complete? state)
;        (if (new-state? state)
;          (improve-design appl method state)
;          (design-succeeds state))
;        (extend-design appl method state)))
;    (state-fails state)))

(defmethod DESIGN-FROM-STATE (appl (method propose&improve) (state p&i-design-state))
  (if (state-feasible? state)
    (if (violated-constraints state)
      (state-fails state)
      (if (state-complete? state)
        (cond ((state-complete? (predecessor state))
               (cond ((and (cheaper-cost? (state-cost state)
                                       (state-cost (predecessor state)))
                        (> (improvement-margin (state-cost (predecessor state)) (state-cost state))
                           *cost-improvement-measure*))
                      (improve-design appl method state))
                     (t
                      (local-best-achieved state))))
                      
              (t
               (when (new-state? state)
                 (print-with-spaces (state-depth state)
                                    "Design is now complete..trying to improve solution..."))
               (improve-design appl method state)))
        (extend-design appl method state)))
    (state-fails state)))

(defun local-best-achieved (state)
  state
  (print-with-spaces 2 ;;; (state-depth state) 
                     "Local optimal design has been reached. Cost of final state is ~s"
                     (state-cost state))
  
  (throw 'exit t))



(defmethod IMPROVE-DESIGN (appl (method design-method) (state p&i-design-state))
 ;;(break)
  (setf (state-foci state)  (collect-state-foci appl method state :improve) 
        (new-state? state) nil)
  (loop for result = (design-from-context appl method state :improve)
        while result))

(defmethod collect-state-foci (appl (method design-method) (state p&i-design-state)
                                    (context (eql :improve)))
  nil)

(defmethod COLLECT-OPERATORS-APPLICABLE-TO-FOCUS (appl (method design-method)
                                                       (state design-state)
                                                       (focus parameter)
                                                       (context (eql :improve)))
  nil)


(defmethod EVALUATE-STATE-INTERNAL :after (appl (method design-method)
                                                (state p&i-design-state))
  (when (state-complete? state)
      (multiple-value-bind (cost model)
                           (evaluate-state-cost appl method state)
        (setf (state-cost state) cost
              (state-detailed-cost state) model))))

(defmethod SELECT-DESIGN-STATE (appl (method propose&improve) space)
  appl method
  (cond (space
         (let* ((newest (car space))
                space-old new-cheapest
                (s (if (or (not (state-complete? newest))
                           (not (state-complete? (predecessor newest))))
                     (if (and (state-feasible? newest)
                              (not (violated-constraints newest)))
                       newest
                       (predecessor newest))
                     ;;;both newest and his predecessor are complete.....
                     (let ((space-new (filter space #'(lambda (s)
                                                        (= (state-depth s) (state-depth newest))))))                                    
                       (setf space-old (filter space #'(lambda (s)
                                                        (not (= (state-depth s) (state-depth newest)))))
                             space-new (filter space-new #'(lambda (s)
                                                             (and (state-feasible? s)
                                                                  (not (violated-constraints s))))))
                       (cond (space-new
                              (setf new-cheapest (first (sort space-new #'cheaper-state?)))
                              (cond ((cheaper-state? new-cheapest (predecessor new-cheapest))
                                     (setf *design-space*
                                           (cons new-cheapest space-old))
                                     new-cheapest)
                                    (t
                                     (predecessor newest)))))))))
           
           (lines 2)
           (print-with-spaces 2 
                              "Current state is now ~s" s)
           s))
        (t
         (print-with-spaces 2 
                            "Cannot select any state"))))
  