;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: ocml;   -*-

(in-package "OCML")

(defmacro def-class (name superclasses &optional instance-var  documentation
                                 class-slots &rest relation-spec)
  `(define-domain-class ',name ',superclasses ',instance-var  ',documentation
                              ',class-slots ',relation-spec))

(defmacro def-domain-class (name superclasses &optional instance-var  documentation
                                 class-slots &rest relation-spec)
  `(define-domain-class ',name ',superclasses ',instance-var  ',documentation
                              ',class-slots ',relation-spec))

(defmacro def-domain-instance (name parent &optional documentation
                                    slots)
  `(define-domain-instance ', name ',parent ',documentation ',slots))

(defmacro def-instance (name parent &optional documentation
                                    slots)
  `(define-domain-instance ', name ',parent ',documentation ',slots))


(defmacro def-relation (name  &optional schema documentation &rest options)
  `(define-relation-internal ',name  ',schema ',documentation ',options))

(defmacro def-operator (name  &optional schema documentation &rest options)
  `(define-relation-internal ',name  ',schema ',documentation ',options))


;(defmacro def-function (name  &optional schema documentation &rest options)
;  `(define-function-internal ,name  ,schema ,documentation ,@options))

(defmacro def-function (name  schema &optional arrow var  documentation &rest options)
  `(define-function-internal ,name  ,schema ,arrow ,var ,documentation ,@options))

(defmacro def-procedure (name  &optional schema documentation &rest options)
  `(define-procedure-internal ,name  ,schema ,documentation ,@options))


(defmacro def-rule (name &optional documentation &rest args)
  `(def-rule-internal ',name ',documentation ',args))



(defmacro def-task (name superclasses &optional instance-var  documentation
                         class-slots &rest relation-spec)
  `(def-task-internal ',name ',superclasses ',instance-var  ',documentation
                              ',class-slots ',relation-spec))

;;;;;(defmacro def-task-role (role task  &optional expression)
 ;;;; `(def-role-internal ',role ',task ',expression))

;;;(defmacro def-method-role (role task  &optional expression)
 ;;; `(def-role-internal ',role ',task ',expression method))




(defmacro def-expression (name &optional documentation assignment expression)
  `(def-expression-internal ',name ',documentation ',assignment ',expression))

(defmacro def-ontology (name &optional documentation &rest options)
  `(def-ontology-internal ',name ',documentation ',options))

(defmacro ensure-ontology (name &optional type pathname)
  `(ensure-ontology-internal ',name ',type ',pathname))




(defmacro def-relation-mapping (relation direction documentation &rest clauses)
  `(def-rel-mapping-internal ',relation ',direction ',documentation ',clauses))

;;;DEF-UPWARD-CLASS-MAPPING ---This creates instances of method-class corresponding
;;;to instances of domain class
(defmacro def-upward-class-mapping (domain-class 
                                    method-class 
                                    &key (name-function #'default-upward-naming-function)
                                         (explicit-link t)
                                         (mapping-relation *default-mapping-relation*))
  `(def-upward-class-mapping-internal ',domain-class ',method-class ,name-function
     ,explicit-link ',mapping-relation))

;;;TELL
(defmacro tell (exp &optional documentation)
  `(tell1 ',exp ,documentation))

(defun tell1 (exp &optional documentation env &aux new-form fexp)
  (multiple-value-bind (pred-structure args original-form)
      (parse-assertion exp env)
    (setf new-form (cons (name pred-structure) args))
    (when *trace-assertions*
      (print-with-spaces (1+ *task-level*) "Asserting ~S" new-form))
    (cond ((setf fexp (downward-add-exp pred-structure))
           (trigger-downward-mapping-exp fexp args))
          ((slotp pred-structure)
           (maybe-add-slot-assertion pred-structure args original-form))
          ((get-domain-class (name pred-structure))
               ;;;; (= 1 (length args)))
           (define-domain-instance  (car args) (car original-form)
             (or documentation "")
             (pairify (cdr args))))
          (t
           (maybe-add-relation-instance pred-structure args original-form documentation)))
    new-form))

;;;TELL1
;(defun tell1 (exp &optional documentation env &aux new-form)
;  (multiple-value-bind (pred-structure args original-form)
;      (parse-assertion exp env)
;    (setf new-form (cons (name pred-structure) args))
;    (when *trace-assertions*
;      (print-with-spaces (1+ *task-level*) "Asserting ~S" new-form))
;    (cond ((slotp pred-structure)
;           (maybe-add-slot-assertion pred-structure args original-form))
;          ((get-domain-class (name pred-structure))
;               ;;;; (= 1 (length args)))
;           (define-domain-instance  (car args) (car original-form)
;             (or documentation "")
;             (pairify (cdr args))))
;          (t
;           (maybe-add-relation-instance pred-structure args original-form documentation)))
;    new-form))

(defmacro unassert-and-tell (exp-to-replace exp  &optional documentation)
  `(replace1 ',exp-to-replace ',exp ,documentation))

(defun replace1 (exp-to-replace exp  &optional env documentation)
  (unassert1 exp-to-replace env :no-checks)
  (tell1 exp documentation  env))


(defun add-assertion (predicate args)
  (tell1 (cons predicate args)))

(defun add-multiple-assertions (predicate args)
  (loop for arglist in args
        do
        (add-assertion predicate arglist)))

(defun replace-assertion (predicate args)
  (let((relation (get-relation predicate)))
    (when relation
      (unassert1 (cons predicate
                       (schema relation))))
    (add-assertion predicate args)))





;;;UNASSERT ---Deletes all assertions matching <exp>, which has the format 
;;;(<pred><term1.>....)
;;;Terms can contain variables
(defmacro unassert (exp)
  `(unassert1 ',exp))


;;;UNASSERT1
(defun unassert1 (exp &optional env no-checks &aux class fexp)
  (multiple-value-bind (pred-structure args )
      (parse-assertion-to-delete exp env)
    (when pred-structure
      (cond ((setf fexp (downward-remove-exp pred-structure))
             (trigger-downward-mapping-exp fexp args))
	    ((slotp pred-structure)
             (maybe-delete-slot-assertion pred-structure args exp no-checks))
            ((setf class (get-domain-class (car exp)))
             (if (= 1 (length args))
		 (remove-domain-instance-gen  (car args) (car exp) class)
                 (remove-instances-matching-spec class (car args)(cdr args) (name pred-structure))))
	    (t
             (remove-relation-instance-gen pred-structure args
                                       (some #'variable? args)))))))


;;;COMPILE-RULE-PACKETS ---Compiles the forward chaining rules in the specifies packets.
;;;By default it compiles all rules in all packets
(defun compile-rule-packets (&optional (packets :all))
  (compile-fc-rules  (if (eq packets :all)
                         (merge-rules (all-rule-packets))
                         (merge-rules (mapcar #'get-rule-packet packets)))))

(defun COMPILE-ALL-FUNCTIONS-&-PROCEDURES (&optional verbose force-p)
  (compile-all-funs&procs verbose force-p))


;;;RUN ---Invokes the forward chainer
(defun run (&optional (packets :all))
  (run-fc-rules packets))


;;;EXECUTE-APPLICATION
(defun execute-application (appl)
  (setf *current-application* appl)
  (apply-method (find-instance (the-slot-value appl 'uses-method) 
                               'problem-solving-method)
                appl))

(defmethod apply-method (method appl)
  (ocml-warn "No specific APPLY-METHOD defined for method ~s on application ~s"
             method appl))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;Simple Tracing


;;;FORWARD CHAINING
(defun trace-fc ()
  (trace-assertions)
  (setf *trace-fc* t))

(defun untrace-fc ()
  (setf *trace-fc* nil))

;;;;;;;;;;;;;;;;;;;;;

;;;BACKWARD CHAINING
(defmacro spy (&rest preds)
  `(if ', preds ;;;;;(eq preds :all)
      (unless (eq *spied-predicates* :all)
	(Setf *spied-predicates*(union ', preds *spied-predicates*)))
      (setf *spied-predicates* :all)))


(defmacro  nospy (&rest preds)
  `(if ',preds
      (if (listp *spied-predicates*)
          (setf *spied-predicates* (Set-difference *spied-predicates* ',preds))
          ;;;We are currently spying all predicates, and the argument to nospy specifies
          ;;;which ones shouldn't be spied anymore.  This means we should spy all the
          ;;;existing predicates not explicitly listed in preds. This is too much effort.
          ;;;Let's just do a global nospy
          (setf *spied-predicates* nil))
      (Setf  *spied-predicates* nil)))


;;;;;;;;;;;;;;;;;;;;;
;;;ASSERTIONS
(defun trace-assertions ()
  (setf *trace-assertions* t))

(defun untrace-assertions ()
  (setf *trace-assertions* nil))

;;;FUNCTIONS
(defmacro trace-function (&rest funs)
  `(dolist (fun ',funs)
     (trace-this-fun (get-function fun))))

(defmacro untrace-function (&rest funs)
  `(dolist (fun ',funs)
     (untrace-this-fun (get-function fun))))

(defun untrace-all-functions ()
  (maphash #'(lambda (key fun)
               (declare (ignore key))
               (setf (fun-trace-on? fun)nil))
           *defined-functions*))

;;;UNTRACE-ALL
(defun untrace-all ()
  
  (untrace-assertions)
  (untrace-all-functions)
  (nospy)
  (untrace-fc))
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;OCML-EVAL ---Top level macro to evaluate a control or function expression
(defmacro ocml-eval (term &optional env)
  `(ocml-eval-gen ',term ,env))

;;;OCML-EVAL-GEN --Top level function to evaluate a control or function expression
(defun ocml-eval-gen (term &optional env)
  (cond ((consp term)
	 (if (procedure? (car term))
             (procedure-eval term env)
             (ocml-eval-fun term env)))
	((variable? term)
         (ocml-eval-var-or-error term env))
        (t
         term)))

;;;OCML-EVAL-FUN-EXPR- This evaluates a function expression (not a 
;;;control expresssion!)
(defun ocml-eval-fun-exp (fun-exp  &optional env)
  (ocml-eval-fun fun-exp  env))
  

;;;OCML-CALL  ---Invokes a function on some arguments.
(defun ocml-call (fun &rest args)
  (ocml-eval-fun-term fun args))

(defun ocml-apply (fun  args &optional env)
  (ocml-eval-fun-term fun args env))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun direct-instances (class)
  (let ((class-structure (get-domain-class class)))
    (if class-structure
        (get-direct-instances class-structure)
	(error "~S is not a domain class" class))))

(defun all-instances (class)
  (append (direct-instances class)
          (mapcan* #'(lambda (x)
                       (all-instances (name x)))
                  (direct-subclasses class (get-domain-class class)))))

;;;INSTANCE-OF? ---True if name is an instance (not just a direct instance!!!) of  class.  
(defun instance-of? (name class)
  (find-instance-of-class (get-domain-class class)name))

  ;(let ((class-structure (get-domain-class class))
;        (instance (find-instance name)))
;    (cond ((and class-structure instance)
;           (typep instance class))
;          (class-structure
;           (error "~S is not a known domain instance" name))
;          (t
;           (error "~S is not a known domain class" class)))))


;;;***************************************************************

;;;HOLDS?  ---This is useful to check whether an expression is true in the OCML KB.
;;;It returns NIL if the expression is false.  Otherwise it returns T if no binding
;;;was created or the resulting environment.
(defun holds? (exp &optional env)
  (Let ((result (ask-top-level exp :env env)))
    (cond ((eq  result :fail)
           nil)
          ((null result)
           t)
          (t
           result))))

;;;FINDALL --Returns all the correct matches for a goal and then returns the list of
;;;the instantiations of <exp> in the resulting environments.  FINDALL returns
;;;NIL (the empty list) if no match is found
(defun findall (exp goal &optional env)
  (Let ((envs (ask-top-level goal :all t :env env)))
    (if (eq envs :fail)
        nil
	(mapcar #'(lambda (env)
                  (instantiate exp env))
              envs))))

;;;SETOFALL ---Like findall, but makes sure there are no duplicates
(defun setofall (exp goal &optional env)
  (remove-duplicates (findall exp goal env)
                     :test #'equal))
                     

;;;FINDANY  --Find one (and only one)  correct match for a goal and then returns 
;;;the instantiation of <exp> in the resulting environment (+ the continuation).
;;;FINDANY returns :NOTHING if no match is found
(defun findany (exp goal &optional env)
  (multiple-value-bind (nenv cont)
                       (ask-top-level goal :env env)
    (findany-aux exp goal env nenv cont)))

(defun findany-gen (exp goal &optional env)
  (loop with result
        with cont
        initially (multiple-value-setq (result cont)
                    (findany exp goal env))
        
        do
        (cond((eq result :nothing)
              (return :nothing))
             (t
              (print result)
              (multiple-value-setq (result cont)
                    (funcall cont))))))
              
;;;ASK
(defmacro ask (query &optional all? env)
  `(ask-top-level ',query :all ',all? :env ,env :query-mode t))

;;;ESTABLISH = ASK, GET result, and  TELL it
(defmacro establish (query)
  `(establish1  ',query))

(defun establish1 (goal &optional env)
  (when (member (car goal)
                  '(exists not or))
    (error "Can't assert a pattern such as ~S" goal))
  ;(setf goal (cons (instantiate (car goal) env)
;                   (cdr goal)))
  (let ((new-env (ask-top-level goal :env env)))
    (cond ((eq new-env :fail)
           :fail)
          (t
           (tell1 goal nil new-env)
           new-env))))

;;;ESTABLISH-ALL
(defun establish-all (goal &optional env)
  (when (member (car goal)
                  '(exists not or))
    (error "Can't assert a pattern such as ~S" goal))
  ;(setf goal (cons (instantiate (car goal) env)
;                   (cdr goal)))
  (let ((new-envs (ask-top-level goal :all t :env env)))
    (cond ((eq new-envs :fail)
           :fail)
          (t
           (if (eq (car goal) 'and)
                 (dolist (exp (cdr goal))
                   (dolist (env new-envs)
                     (tell1 exp nil env)))
		 (dolist (env new-envs)
                     (tell1 goal nil env)))
           new-envs))))


;;;ASK-TOP-LEVEL
(defun ask-top-level (goal &key all env query-mode compiled)    
  (cond ((variable? goal)
         (multiple-value-bind (flag value)
                              (ocml-eval-var goal env)
           (cond (flag
                  (unless (listp value)
                    (error "A goal must be a list..when parsing ~s" value))
                  (ask-internal value all env query-mode compiled ))
                 (t
                  (error "An unbound variable such as ~A is not a legal goal"
                         value)))))
        (t (unless (listp goal)
             (error "A goal must be a list..when parsing ~s" goal))
           (ask-internal goal all env query-mode compiled ))))

;;;----------------------------------------------------

;;;SLOT-VALUES  ---Useful to get values of slots of domain instances (rather than using
;;;the more generic ask primitive)
(defun slot-values (name slot)
  (let ((instance (find-instance name)))
    (if instance
        (get-slot-values instance slot)
        (error "~S is not a domain instance" name))))
    

;;;THE-SLOT-VALUE
(defun the-slot-value (name slot)
  (let ((values (slot-values name slot)))
    (if values
      (car values)
      :nothing)))
    

(defun add-slot-value (name slot value)
  (let ((instance (find-instance name)))
    (if instance
        (add-slot-value-to-instance instance slot value)
        (error "~S is not a domain instance" name))))

(defun clear-slot-values (name slot)
  (let ((instance (find-instance name)))
    (if instance
        (remove-local-slot-values instance slot )
        (error "~S is not a domain instance" name))))

(defun set-slot-value (name slot value)
  (let ((instance (find-instance name)))
    (cond(instance
           (remove-local-slot-values instance slot )
           (add-slot-value name slot value))
         (t
           (error "~S is not a domain instance" name)))))
  
  
 

